/*
 * ============================================================================
 * Licensed Materials - Property of IBM
 *  WebSphere sMash
 *
 * (C) Copyright IBM Corp. 2008  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 * ============================================================================
 */
 
 import zero.core.etags.Etag
 import zero.ide.services.AppUtils
 import zero.ide.services.FileUtils
 import zero.ide.services.ExtensionFileFilter
 import zero.ide.services.HiddenFileFilter
 import zero.ide.services.TemplateGenerator
 import zero.cli.tasks.utils.ToolsUtils
 import zero.core.utils.VDirectory
 import zero.core.context.GlobalContext;
 import zero.core.context.GlobalContextURIs;

 //
// getETag() returns the ETag for a file as an unquoted string.
 //
 def getETag(target)
 {
  def modified = target.lastModified().toString();
  return Etag.computeHashedValue(modified);
 }
    
 def onUpdate()
 {
	def componentType = "zwidget";
	def contextRoot=GlobalContext.zget('/config/contextRoot');
	if (contextRoot==null) {
		contextRoot='';
 	}
	
	def uriPrefix = contextRoot + '/resources/applications';
	def requestUri = request.uri[];
 	if (!requestUri.startsWith(uriPrefix)) {
 		// onUpdate is currently only valid within the application
 		request.status = 405
 		return
 	}   
 	
 	def ifMatch = zget("/request/headers/in/If-Match");
 	def ifNoneMatch = zget("/request/headers/in/If-None-Match");
 	
 	//
 	// remove quotes from etag value
 	//
 	if (ifMatch != null)
 		ifMatch = ifMatch[1..-2];

 	def isCreating = ifNoneMatch != null && ifNoneMatch.equals("*");    
 	
 	if (isCreating)
 	{
 		//
 		// can't be creating a new file and matching against an existing
 		// version of a file at the same time
 		//
 		if (ifMatch != null)
 		{
 			request.status = 412;
 			return;
 		}

 		handleCreate(componentType);
 	}
 	else
 	{
 		// Can't do an update through this method
 		request.status = 412;
 		return;
 	}
 }

 def fullyQualify(fileName)
 {
     def apphome = AppUtils.getApphome(request.params.applicationsId[])
     File f = new File(apphome, fileName)
     return f
 }

 def dprintln(String value)
 {
 	System.out.println(value)
 }

 def dumpInfo(componentType)
 {
 	def appId = request.params.applicationsId[]
 	dprintln("appId="+appId)
 	def compId = zget("/request/params/"+componentType+"/Id")
 	dprintln("compId="+compId)
 	def templateName = request.params.template[0]
 	if (templateName)
 		dprintln("template="+templateName)
 	def pathInfo = event.pathInfo[];
 	dprintln("pathInfo="+pathInfo);	
 }
  

 /**
  * @success 200 returns the component metadata { commandRunning: [true|false] }
  * @error 404 if the
  */  
 def handleCreate(componentType)
 { 	    
    // The first directory in the path comes in as the ID    
    //dumpInfo(componentType)
	def pathName1 = zget("/request/params/"+componentType+"Id")    
    def pathName2 = event.pathInfo[]

    if (!pathName1) {    	
 		request.status = 404 
 		return
    }    	
    if (pathName2!=null && pathName2.length()>0)
      pathName1 += pathName2;
	
    def relativeTargetName = "app/zwidgets/"+pathName1;  // +pathName2
    this.resultUri = "resources/applications/"+request.params.applicationsId[]+"/files/"+relativeTargetName+"/"+request.params.zwname[0]+".js"
    
    def targetFile = fullyQualify(relativeTargetName)		
    if (targetFile.exists())
    {
 		// File already exists!
 	    request.status = 412;
 	    return;
    }
    
    def templateName = request.params.template[0]
    if ((!templateName) || (templateName.length() == 0))
    {	   
       // template required       
 	   request.status = 404;
 	   return;
    }
    else
    {             	
    	// Template file comes from the zero.appbuilder/app/templates directory   
    	def templateDir = VDirectory.getVFile("/app/templates/"+componentType+"/"+templateName)  
    	if ((templateDir == null) || (!templateDir.exists()) || (!templateDir.isDirectory()))
    	{    		
    		request.status = 404
    		return
    	}		    
   		generateDirectory(targetFile, relativeTargetName, templateDir, templateName)
    		
    	response = targetFile
    	request.headers.out.Location = resultUri
    	request.headers.out.ETag = "\"${getETag(targetFile)}\"".toString();
    	request.view = 'JSON'
    	request.json.output = [ 'url' : resultUri ]
    	request.status = 200
    	render()   
    }
 }
  
 def copyFile(templateFile, targetFile) 
 {
 	ToolsUtils.copyFile(templateFile, targetFile)
 } 
 
 def generateDirectory(targetDir, relativeTargetName, templateDir, templateName) 
 {
 	 TemplateGenerator tg = new TemplateGenerator(templateDir, targetDir)
 	  	 
 	 tg.setContextVariable(TemplateGenerator.templateNameVar, templateName)
 	 
 	 def appName = zget("/storage/applications#"+request.params.applicationsId[]+"/name") 	 
 	 tg.setContextVariable(TemplateGenerator.appNameVar,appName)
 	 
 	 def zwpackage = request.params.zwpackage[0]
 	 tg.setContextVariable("zwPackage",zwpackage)
 	 
 	 // def zwpackagedir = relativeTargetName.substring(relativeTargetName.indexOf("widgets/")+8,relativeTargetName.lastIndexOf("/"))
 	 def zwpackagedir = relativeTargetName.substring(relativeTargetName.indexOf("widgets/")+8)
 	 tg.setContextVariable("zwPackageDir",zwpackagedir) 	 
 	 
 	 def zwname = request.params.zwname[0]
 	 tg.setContextVariable(TemplateGenerator.nameVar,zwname) 	 
     
 	 // Invoke the generator, telling it to replace all occurrences of the template name with 
 	 tg.generate()
 }
 
 def createWithoutTemplate(targetFile)
 {	
     targetFile.getParentFile().mkdirs()
     
     def output = new FileOutputStream(targetFile)
     
     output.flush()
     output.close()        
 }
