/*
 * ============================================================================
 * Licensed Materials - Property of IBM
 *  WebSphere sMash
 *
 * (C) Copyright IBM Corp. 2008  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 * ============================================================================
 */


// 
// File: /app/scripts/FileTemplateHandler.groovy
// 
// Author: Brett King (brettk)
// 
// This script includes the RESTful API for the IDE's template-based file
// creation system
//


import zero.core.etags.Etag
import zero.ide.services.AppUtils
import zero.ide.services.FileUtils
import zero.ide.services.ExtensionFileFilter
import zero.ide.services.HiddenFileFilter
import zero.ide.services.TemplateGenerator
import zero.cli.tasks.utils.ToolsUtils
import zero.core.utils.VDirectory
import zero.appbuilder.resources.MessageHandler
import zero.core.utils.ClientMessageFormatter
import zero.ide.services.IWidget;
import zero.ide.services.IWidgetInfo;
import zero.ide.services.IWidgetComponent;
import zero.ide.services.IWidgetComponentInfo;
import zero.core.context.GlobalContext;
import zero.core.context.GlobalContextURIs;

//
//getETag() returns the ETag for a file as an unquoted string.
//
def getETag(target)
{
 def modified = target.lastModified().toString();
 return Etag.computeHashedValue(modified);
}
   
def onUpdate(componentType)
{
	def contextRoot=GlobalContext.zget('/config/contextRoot');
	if (contextRoot==null) {
		contextRoot='';
 	}
	
	def uriPrefix = contextRoot + '/resources/applications';
	def requestUri = request.uri[];
 	if (!requestUri.startsWith(uriPrefix)) {
		// onUpdate is currently only valid within the application
		request.status = 405
		return
	}   
	
	def ifMatch = zget("/request/headers/in/If-Match");
	def ifNoneMatch = zget("/request/headers/in/If-None-Match");

	//
	// remove quotes from etag value
	//
	if (ifMatch != null)
		ifMatch = ifMatch[1..-2];

	def isCreating = ifNoneMatch != null && ifNoneMatch.equals("*");    
	
	if (isCreating)
	{
		//
		// can't be creating a new file and matching against an existing
		// version of a file at the same time
		//
		if (ifMatch != null)
		{
			request.status = 412;
			return;
		}

		handleCreate(componentType);
	}
	else
	{
		// Can't do an update through this method
		request.status = 412;
		return;
	}
}


def dprintln(String value)
{
	System.out.println(value)
}

def dumpInfo(componentType)
{
	def appId = request.params.applicationsId[]
	dprintln("appId="+appId)
	def compId = zget("/request/params/"+componentType+"/Id")
	dprintln("compId="+compId)
	def templateName = request.params.template[0]
	if (templateName)
		dprintln("template="+templateName)
	def pathInfo = event.pathInfo[];
	dprintln("pathInfo="+pathInfo);	
}
 

/**
 * @success 200 returns the component metadata { commandRunning: [true|false] }
 * @error 404 if the
 */  
def handleCreate(componentType)
{ 	    
   // The first directory in the path comes in as the ID    
   //dumpInfo(componentType)
   def pathName1 = zget("/request/params/"+componentType+"Id")      
   def pathName2 = event.pathInfo[]
   
   if (!pathName1)
   {
		request.status = 404 
		return
   }   
   if (!pathName2) {
	   // When creating "other" files, we can start at the root of the 
	   // app, so there is not pathInfo.
	   if (componentType.equalsIgnoreCase("other")) 
	   {
		   pathName2 = ""
	   }
	   else
	   {
		   request.status = 404
		   return
	   }
   }
   
   def apphome = AppUtils.getApphome(request.params.applicationsId[]);
   def fullName = pathName1+pathName2;
   this.resultUri = "resources/applications/"+request.params.applicationsId[]+"/files/"+fullName;  
   File targetFile = new File(apphome, fullName);
   if (!targetFile.isDirectory() && !targetFile.exists() && targetFile.path.endsWith(".html")) {
	   IWidgetComponentInfo iwcInfo = IWidget.getComponentInfo(apphome,targetFile.toURL().toString());
	   if (iwcInfo.isIWidgetComponent && iwcInfo.widgetFile.exists()) {
		   IWidget iwidget = new IWidget(iwcInfo.appHome, iwcInfo.widgetFile);
           if (iwidget.componentExists(iwcInfo.componentMode)) {
        	   // trying to CREATE a Component that already existss
        	   request.status = 412;
        	   request.view = 'error'
        	   request.headers.out.'Content-Type' = 'application/json'
        	   ClientMessageFormatter messageFormatter = new ClientMessageFormatter("zero/appbuilder/core/resources/consolemessages");
        	   request.error.message = messageFormatter.getMessage('FILE_ALREADY_EXISTS', targetFile.absolutePath);
        	   render()
        	   return;         	
           }
           String contents=' <!-- '+iwcInfo.componentName+' -->\n';
           iwidget.updateComponentFromHTML(iwcInfo.componentMode, contents);
           iwidget.save();

           File iWidgetFile=iwcInfo.widgetFile;
           
           request.headers.out.ETag = "\"${getETag(iWidgetFile)}\"".toString();
           request.headers.out.Location = resultUri
           request.view = 'JSON'
           request.json.output = [ 'url' : resultUri ]
           request.status = 200
           render()
           return;
	   }
   } 
	
   if (targetFile.exists())
   {
		// File already exists!
	    request.status = 412;
		request.view = 'error'
		request.headers.out.'Content-Type' = 'application/json'
		ClientMessageFormatter messageFormatter = new ClientMessageFormatter("zero/appbuilder/core/resources/consolemessages");
		request.error.message = messageFormatter.getMessage('FILE_ALREADY_EXISTS', targetFile.absolutePath);
		render()
	    return;
   }
   
   def templateName = request.params.template[0]   
   if ((!templateName) || (templateName.equalsIgnoreCase("* None *")))
   {	   
	   createWithoutTemplate(targetFile)
   }
   else
   {	      
	   // Template file comes from the zero.appbuilder/app/templates directory   
	   def templateFile = VDirectory.getVFile("/app/templates/"+componentType+"/"+templateName)  
	   if ((templateFile == null) || (!templateFile.exists()))
	   {
		   request.status = 404
		   return
	   }		
   
	   if (templateFile.isDirectory())
		   generateDirectory(targetFile, templateFile, templateName)
	   else {
		   generateFile(targetFile, templateFile, templateName)		   		   		   
	   }
   }
   request.headers.out.ETag = "\"${getETag(targetFile)}\"".toString();
   request.headers.out.Location = resultUri
   request.view = 'JSON'
   request.json.output = [ 'url' : resultUri ]
   request.status = 200
   render()   

}
 
def copyFile(templateFile, targetFile) 
{
	ToolsUtils.copyFile(templateFile, targetFile)
} 

def generateDirectory(targetDir, templateDir, templateName) 
{
	 TemplateGenerator tg = new TemplateGenerator(templateDir, targetDir)
	 tg.setContextVariable("templateName", templateName)
	 def appName = zget("/storage/applications#"+request.params.applicationsId[]+"/name")
	
	 tg.setContextVariable("appName",appName)
			 
	 tg.generate()
}

def generateFile(targetFile, templateFile, templateName) 
{
	 TemplateGenerator tg = new TemplateGenerator()
	 tg.setContextVariable("templateName", templateName)
	 def appName = zget("/storage/applications#"+request.params.applicationsId[]+"/name")
	 
	 tg.setContextVariable("appName",appName)
			 
	 tg.generate(templateFile, targetFile)
}

def createWithoutTemplate(targetFile)
{	
    targetFile.getParentFile().mkdirs()
    
    def output = new FileOutputStream(targetFile)
    
    output.flush()
    output.close()        
}
